@@ -7,7 +7,7 @@
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
- * as published by the Free Software Foundation; either version 2    
+ * as published by the Free Software Foundation; either version 3    
  * of the License, or any later version.                             
  *                                                                   
  *                                                                   
@@ -18,40 +18,22 @@
  *                                                                   
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software       
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston,            
- * MA  02111-1307, USA.                                              
+ * Foundation, Inc., 51 Franklin Street, Boston,            
+ * MA  02110-1301, USA.                                              
  ****/
 
 
-#include <X11/extensions/XTest.h>
-
 #include "libxnee/xnee.h"
 #include "libxnee/print.h"
 #include "libxnee/xnee_record.h"
 #include "libxnee/xnee_replay.h"
 #include "libxnee/xnee_sem.h"
 #include "libxnee/xnee_buffer.h"
+#include "libxnee/xnee_threshold.h"
+#include "libxnee/xnee_setget.h"
 #include "libxnee/xnee_resolution.h"
 
 
-/* 
- * internal use only 
- */
-/**************************************************************
- *                                                            *
- * xnee_replay_buffer_status                                  *
- *                                                            *
- *                                                            *
- **************************************************************/
-int
-xnee_replay_buffer_status (xnee_data* xd, int data_type, int nr);
-int
-xnee_replay_buffer_max_diff (xnee_data* xd, int type);
-int
-xnee_replay_buffer_min_diff (xnee_data* xd, int type);
-int
-xnee_replay_buffer_tot_diff (xnee_data* xd, int type);
-
 
 /**************************************************************
  *                                                            *
@@ -154,7 +136,7 @@ xnee_replay_buffer_handle (xnee_data* xd, int data_type, int nr, Bool rec_or_rep
   /*
    * protect the buffer by waiting for the buffer semaphore
    */
-  xnee_sem_wait (xd, xd->buf_sem);
+  xnee_sem_wait(xd, xd->buf_sem);
 
   /*
    * From where did we get the data?
@@ -212,16 +194,32 @@ xnee_replay_buffer_handler (xnee_data* xd,
 			    int data_nr,
 			    Bool rec_or_rep)
 {
+
+  /*
+   * We don't use device event while synching 
+   *
+  if ( (data_type==XNEE_EVENT) &&
+       ( (data_nr>=KeyPress) || (data_nr<=MotionNotify)) )
+    {
+      return;
+    }
+   */
+
+      
   xnee_verbose((xd,"---> xnee_replay_buffer_handler "));
   if (rec_or_rep==XNEE_RECEIVED) 
-    xnee_verbose((xd," XNEE_RECEIVED \n"));
+    {
+      xnee_verbose((xd," XNEE_RECEIVED \n"));
+    }
   else
-    xnee_verbose((xd," XNEE_REPLAYED \n"));
+    {
+      xnee_verbose((xd," XNEE_REPLAYED \n"));
+    }
 
   /*
    * protect the buffer by waiting for the buffer semaphore
    */
-  xnee_sem_wait (xd, xd->buf_sem);
+  xnee_sem_wait(xd, xd->buf_sem);
 
 
   /*
@@ -238,6 +236,22 @@ xnee_replay_buffer_handler (xnee_data* xd,
 	    ( data_type == XNEE_EVENT ) && 
 	    ( data_nr   <= ButtonRelease ) ))
 	{
+	    if ( ( data_nr >= KeyPress )
+		 && 
+		 ( data_nr <= MotionNotify )
+		 && 
+		 xnee_is_replayer(xd))
+	    {
+		xnee_verbose((xd,"---  xnee_replay_buffer_handler : ignonring recorded data "));
+
+	      /*
+	       * release the protecting semaphore the buffer
+	       */
+	      xnee_sem_post (xd, xd->buf_sem);
+	      return;
+
+	    }
+	    
 	  if (data_nr==161)
 	    {
 	      fprintf (stderr, 
@@ -261,7 +275,8 @@ xnee_replay_buffer_handler (xnee_data* xd,
 	  if (xd->meta_data.cached_max == 
 	      xd->data_buffer[data_type][data_nr])
 	    {
-	      /* don't get it .... please read the above read again */
+	      /* don't get it?
+		 .... please read the above read again */
 	      xd->meta_data.cached_max=-1;
 	    }
 
@@ -356,18 +371,15 @@ xnee_update_buffer_cache(xnee_data *xd)
   /*
    * protect the buffer by waiting for the buffer semaphore
    */
-  xnee_sem_wait (xd, xd->buf_sem);
+  xnee_sem_wait(xd, xd->buf_sem);
   
   for (counter=0 ; counter < MAX_UNSYNC_LOOPS; counter++)
     {
-      xnee_verbose((xd, "-----------------------------------------\n"));
       for ( type=0 ; type< 2 ; type++) 
 	{
 	  int real_min=0;
 	  int real_max=0;
 	  
-	  xnee_verbose((xd, "     synchronize: checking type:%d\n", type));
-	  
 	  real_min=xd->meta_data.cached_min;
 	  real_max=xd->meta_data.cached_max;
 	  
@@ -395,8 +407,6 @@ xnee_update_buffer_cache(xnee_data *xd)
 			     xd->meta_data.cached_max));
 	      xd->meta_data.cached_max=real_max;
 	    }
-	    
-
 	}
     }
   /*
@@ -422,21 +432,24 @@ xnee_hard_update_buffer_cache(xnee_data *xd)
   /*
    * protect the buffer by waiting for the buffer semaphore
    */
-  xnee_sem_wait (xd, xd->buf_sem);
+  xnee_sem_wait(xd, xd->buf_sem);
   
   for (counter=0 ; counter < MAX_UNSYNC_LOOPS; counter++)
     {
-      xnee_verbose((xd, "-----------------------------------------\n"));
       for ( type=0 ; type< 2 ; type++) 
 	{
 	  
 	  real_min = xnee_replay_buffer_min_diff (xd,type);
 	  if (real_min<xd->meta_data.cached_min)
-	    xd->meta_data.cached_min=real_min;
+	  {
+	      xd->meta_data.cached_min=real_min;
+	  }
 
 	  real_max = xnee_replay_buffer_max_diff (xd,type);
 	  if (real_max>xd->meta_data.cached_max)
-	    xd->meta_data.cached_max=real_max;
+	  {
+	      xd->meta_data.cached_max=real_max;
+	  }
 	}
     }
   /*
@@ -459,42 +472,50 @@ xnee_check_buffer_limits (xnee_data *xd)
   int sum_min    = 0 ;
   int tot_diff   = 0 ;
   int diff       = 0 ;
-
-
-  /* HESA HESA HESA TO REMOVE 
-  xnee_hard_update_buffer_cache (xd);
-  */
-  xnee_hard_update_buffer_cache (xd);
-
+  static int upd_ctr = -1 ;
+  
+  if ( ( upd_ctr == -1 )  ||  ( upd_ctr > 10 ) )
+  {
+      xnee_verbose ((xd, " xnee_hard_update_buffer_cache\n"));
+      xnee_hard_update_buffer_cache (xd);
+      upd_ctr=0;
+  }
+  else
+  {
+      xnee_verbose ((xd, " xnee_update_buffer_cache\n"));
+      xnee_update_buffer_cache (xd);
+  }
+  upd_ctr++;
 
   cached_max = xd->meta_data.cached_max;
   cached_min = xd->meta_data.cached_min;
   sum_max    = xd->meta_data.sum_max;
   sum_min    = xd->meta_data.sum_min;
   tot_diff   = xd->meta_data.total_diff;
+
   xnee_verbose ((xd, "---> xnee_check_buffer_limits  button %d   key %d\n", 
 		 xd->button_pressed,
 		 xd->key_pressed
 		 ));
 
-  xnee_verbose ((xd, "---  %02d %02d %02d %02d %02d \n",
+  xnee_verbose ((xd, "---  cached: %02d/%02d  sum_max=%02d   sum_min=%02d   sum_tot=%02d \n",
 		 cached_max ,
 		 cached_min ,
 		 sum_max    ,
 		 sum_min    ,
 		 tot_diff   ));
 
-  if ( cached_max > XNEE_BUFFER_MAX) 
+  if ( cached_max > xnee_get_max_threshold(xd)) 
     {
-      xnee_verbose ((xd, "cached_max %d > XNEE_BUFFER_MAX %d\n",
-		     cached_max , XNEE_BUFFER_MAX));
+      xnee_verbose ((xd, "cached_max %d > xnee_get_max_threshold(xd) %d\n",
+		     cached_max , xnee_get_max_threshold(xd)));
       xnee_replay_printbuffer(xd); 
       diff=cached_max;
     }
-  else if ( cached_min < XNEE_BUFFER_MIN) 
+  else if ( cached_min < xnee_get_min_threshold(xd)) 
     {
-      xnee_verbose ((xd, "cached_min %d < XNEE_BUFFER_MIN %d\n",
-		     cached_min , XNEE_BUFFER_MIN));
+      xnee_verbose ((xd, "cached_min %d < xnee_get_min_threshold(xd) %d\n",
+		     cached_min , xnee_get_min_threshold(xd)));
       xnee_replay_printbuffer(xd); 
       diff=cached_min;
     }
@@ -512,18 +533,19 @@ xnee_check_buffer_limits (xnee_data *xd)
 		     sum_min , XNEE_BUFFER_SUM_MIN));
       xnee_replay_printbuffer(xd); 
       diff=sum_min;
-      }*/
-  else if ( tot_diff > XNEE_BUFFER_TOT_MAX )
+      }
+  */
+  else if ( tot_diff > xnee_get_tot_threshold(xd) )
     {
-      xnee_verbose ((xd, "tot_diff %d > XNEE_BUFFER_TOT_MAX %d\n",
-		     tot_diff , XNEE_BUFFER_TOT_MAX));
+      xnee_verbose ((xd, "tot_diff %d > xnee_get_tot_threshold(xd) %d\n",
+		     tot_diff , xnee_get_tot_threshold(xd)));
       xnee_replay_printbuffer(xd); 
       diff=tot_diff;
     }
   else
     {
       xnee_verbose ((xd, "no diff  %d %d\n",
-		     tot_diff , XNEE_BUFFER_TOT_MAX));
+		     tot_diff , xnee_get_tot_threshold(xd)));
       xnee_replay_printbuffer(xd); 
       diff=0;
     }
