@@ -3,11 +3,12 @@
  *                                                                   
  * Xnee enables recording and replaying of X protocol data           
  *                                                                   
- *        Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Henrik Sandklef 
+ *        Copyright (C) 1999, 2000, 2001, 2002, 2003, 
+ *                      2004, 2008, 2009  Henrik Sandklef 
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
- * as published by the Free Software Foundation; either version 2    
+ * as published by the Free Software Foundation; either version 3    
  * of the License, or any later version.                             
  *                                                                   
  *                                                                   
@@ -54,6 +55,7 @@ xnee_setup_display (xnee_data *xd)
       return XNEE_NOT_OPEN_DISPLAY;
     }
   
+
   
   if (xd->control!=NULL)
     {
@@ -66,6 +68,7 @@ xnee_setup_display (xnee_data *xd)
     }
   
   
+
   if (xd->fake!=NULL)
     {
       XCloseDisplay(xd->fake);
@@ -77,7 +80,7 @@ xnee_setup_display (xnee_data *xd)
     }
 
 
-  xnee_verbose((xd, "Freeing modifier mapping memory %d ", xd->map));
+  xnee_verbose((xd, "Freeing modifier mapping memory %d ", (int)xd->map));
   XFreeModifiermap(xd->map);
 
   xnee_verbose((xd," building modifier map on %d\n", (int)xd->fake)); 
@@ -123,8 +126,17 @@ xnee_setup_display (xnee_data *xd)
 void 
 xnee_set_default_display (xnee_data *xd)
 {
-   xd->display = getenv ((const char*)"DISPLAY");
-   return ;
+  char *tmp;
+  tmp = getenv ((const char*)"DISPLAY");
+  if ( tmp != NULL )
+    {
+      xd->display = strdup(tmp);
+    }
+  else
+    {
+      xd->display = NULL;
+    }
+  return ;
 }
 
 
@@ -144,14 +156,14 @@ xnee_open_display(xnee_data* xd)
   const char *tmp;
 
   tmp = (const char*) ((xd->display!=NULL)?
-		 (xd->display):
-		 NULL_STRING);
-
+		       (xd->display):
+		       NULL_STRING);
+  
   if ( (tmp!=NULL) && (strlen(tmp)==0) )
     {
       tmp=NULL;
     }
-
+  
   xnee_verbose((xd,  "Open display %s \n", 
 		tmp?tmp:"" ));
   
@@ -166,8 +178,7 @@ xnee_open_display(xnee_data* xd)
 			       XDisplayName(tmp));
       return dpy;
     }
-
-
+  
   xnee_verbose((xd, "Display %s = %d\n", 
 		tmp?tmp:"", (int) dpy));
   return dpy;
@@ -186,11 +197,21 @@ xnee_open_display(xnee_data* xd)
 int 
 xnee_add_display_list ( xnee_data* xd, char * disp)
 {
-  char buf[10];
+  char buf[256];
   size_t next;
   size_t len;
   int ret=0;
-  int disp_len=strlen(disp);
+  int disp_len;
+
+
+  if (disp==NULL)
+    {
+      return XNEE_WRONG_PARAMS;
+    }
+
+  disp_len = strlen(disp);
+
+
   xnee_verbose((xd, "xnee_add_display (xd, %s, )\n", disp));
 
   while ( (1!=0) ) {
@@ -220,7 +241,7 @@ xnee_add_display_list ( xnee_data* xd, char * disp)
 
   }
   xnee_verbose((xd, "xnee_add_display_str ... finished\n"));
-  return (0);  
+  return XNEE_OK;  
 }
 
 
@@ -363,10 +384,14 @@ xnee_add_display (Display *dpy, xnee_data* xd)
       xnee_distr *tmp = xd->distr_list;
       xd->distr_list = (xnee_distr *) realloc (xd->distr_list, 
 					       xd->distr_list_size);
-      /*@ignore@*/
-      xd->distr_list = tmp;
-      /*@end@*/
-      return (XNEE_MEMORY_FAULT);
+      if (xd->distr_list==NULL)
+	{
+	  /*@ignore@*/
+	  xd->distr_list = tmp;
+	  /*@end@*/
+	  tmp = NULL;
+	  return XNEE_MEMORY_FAULT;
+	}
     }
   /*@ignore@*/
   XTestGrabControl (dpy, True);
