@@ -460,6 +460,7 @@ xnee_record_dispatch(XPointer xpoint_xnee_data,
 
   XNEE_DEBUG ( (stderr ," --> xnee_record_dispatch()  \n"  ));
 
+
   if ( (data==NULL) || (!data->data) )
     {
       XNEE_DEBUG ( (stderr ," <-- xnee_record_dispatch()  \n"  ));
@@ -770,9 +771,7 @@ xnee_has_record_extension(xnee_data *xd)
 
   if  ( (xd == NULL)
         ||
-        (xd->control == NULL)
-        ||
-        (xd->record_setup == NULL))
+        (xd->control == NULL))
   {
      return XNEE_RECORD_FAILURE;
   }
@@ -781,7 +780,9 @@ xnee_has_record_extension(xnee_data *xd)
                           &xd->record_setup->major_return, 
                           &xd->record_setup->minor_return) == 0 )
     {
-      xnee_print_error ("Record extension missing\n");
+      xnee_print_error ("Record extension missing."
+			" This is not an error in " 
+			PACKAGE "\n");
       ok=XNEE_NO_REC_EXT;
     }
   else
@@ -794,6 +795,48 @@ xnee_has_record_extension(xnee_data *xd)
 }
 
 
+static int 
+xnee_record_from_data_display(xnee_data *xd)
+{
+  int ret_val = 0;
+
+  /*
+  printf ("X info:   %s %d %d %d\n",
+	  xd->x_vendor_name, 
+	  xd->x_version_major, 
+	  xd->x_version_minor, 
+	  xd->x_version_minor_sub);
+  */
+
+  if (xd != NULL)
+    {
+      
+      if (strstr(xd->x_vendor_name, "X.Org") )
+	{
+	  
+	  if ( ( xd->x_version_major == 1 ) &&
+	       ( xd->x_version_minor >= 4 ) )
+	    {
+	      ret_val = 1;
+	    }
+	}
+      else if (strstr(xd->x_vendor_name, "Sun Microsystems") )
+	{
+	  
+	  if ( ( xd->x_version_major == 1 ) &&
+	       ( xd->x_version_minor >= 3 ) )
+	    {
+	      ret_val = 1;
+	    }
+	}
+
+    }
+  
+  return ret_val;
+}
+
+
+
 
 
 /**************************************************************
@@ -806,6 +849,7 @@ int
 xnee_setup_recording(xnee_data *xd)
 {
   int nr_of_ranges=0;
+  Display *context_display;
   nr_of_ranges=xnee_get_max_range(xd);
   xnee_verbose((xd, "--->xnee_setup_recording\n"));
 
@@ -813,9 +857,7 @@ xnee_setup_recording(xnee_data *xd)
        ||
        (xd->control == NULL)
        ||
-       (xd->record_setup == NULL)
-       ||
-       (xd->record_setup->xids == NULL))
+       (xd->record_setup == NULL))
   {
      return XNEE_RECORD_FAILURE;
   }
@@ -834,12 +876,38 @@ xnee_setup_recording(xnee_data *xd)
       xd->record_setup->xids[0] = XRecordFutureClients; 
     }
 
+
+  if ( xnee_record_from_data_display(xd))
+    {
+      /*
+       * From X.org 1.6.0 to ????
+       * 
+       *   there seem to be something strange about 
+       *   the XRecordCreateContext call
+       *   which causes the XRecordEnableContextAsync 
+       *   to fail ... ugly fix, but it works
+       *
+       */
+      fprintf(stderr, "Workaround: Creating context on data display instead of control \n");
+      fprintf(stderr, "            You can ignore this message\n");
+      context_display = xd->data ;
+    }
+  else
+    {
+      context_display = xd->control ;
+    }
+
   xd->record_setup->rContext = 
-    XRecordCreateContext(xd->control, 
+    XRecordCreateContext(context_display, 
 			 xd->record_setup->data_flags, 
 			 xd->record_setup->xids,1, 
 			 xd->record_setup->range_array, nr_of_ranges);
 
+  XFlush(xd->control);
+  XFlush(xd->data);
+  XSync(xd->control, True);
+  XSync(xd->data, True);
+
   /*  printf("%d <=== XRecordCreateContext(%d,%d, %d,%d, %d,%d)\n",
 	 xd->record_setup->rContext,
 	 xd->control, 
@@ -926,6 +994,7 @@ xnee_unsetup_recording(xnee_data *xd)
     {
       xnee_verbose((xd, "---  disabling context %d \n", 
 		    (int)xd->record_setup->rContext));
+  
       (void)XRecordDisableContext(xd->control, xd->record_setup->rContext);
 
       xnee_verbose((xd, "---  freeing context \n"));
@@ -952,6 +1021,7 @@ int
 xnee_record_loop(xnee_data *xd)
 {
    int ret ; 
+   Display *context_display;
 
    if ( (xd==NULL)
         || 
@@ -965,6 +1035,26 @@ xnee_record_loop(xnee_data *xd)
    
    xnee_verbose((xd, " ---> xnee_record_loop()\n"));
   
+  if ( xnee_record_from_data_display(xd))
+    {
+      /*
+       * From X.org 1.6.0 to ????
+       * 
+       *   there seem to be something strange about 
+       *   the XRecordCreateContext call
+       *   which causes the XRecordEnableContextAsync 
+       *   to fail ... ugly fix, but it works
+       *
+       */
+      fprintf(stderr, "Workaround: Creating context on data display instead of control \n");
+      fprintf(stderr, "            You can ignore this message\n");
+      context_display = xd->data ;
+    }
+  else
+    {
+      context_display = xd->control ;
+    }
+
    /* 
     * In case the key pressed to invoke Xnee is not released
     * we wait 1/2 of a second and hopefully it is. If not
@@ -974,10 +1064,10 @@ xnee_record_loop(xnee_data *xd)
    (void) usleep ( XNEE_DELAY_RECORDING );
    /*@ end @*/
    
-   ret = XRecordEnableContext(xd->data, 
-			      xd->record_setup->rContext, 
-			      xd->rec_callback, 
-			      (XPointer) (xd) /* closure passed to Dispatch */);
+   ret = XRecordEnableContextAsync(context_display, 
+				   xd->record_setup->rContext, 
+				   xd->rec_callback, 
+				   (XPointer) (xd) /* closure passed to Dispatch */);
    
    XNEE_RETURN_IF_ERR(ret);
    
@@ -1001,6 +1091,7 @@ int
 xnee_record_async(xnee_data *xd)
 {
    int ret ;
+   Display *context_display;
 
    if ( (xd==NULL)
         || 
@@ -1011,7 +1102,7 @@ xnee_record_async(xnee_data *xd)
        return XNEE_RECORD_FAILURE;
      }
    
-  xnee_verbose((xd, " ---> xnee_record_async()\n"));
+  xnee_verbose((xd, " ---> xnee_record_async() \n"));
   
   /* 
    * In case the key pressed to invoke Xnee is not released
@@ -1026,20 +1117,50 @@ xnee_record_async(xnee_data *xd)
     }
 
 
-  ret = XRecordEnableContextAsync(xd->data, 
-				  xd->record_setup->rContext, 
-				  xd->rec_callback, 
-				  (XPointer) (xd) /* closure passed to Dispatch */);
-  if (!ret)
+  
+  if ( xnee_record_from_data_display(xd))
+    {
+      /*
+       * From X.org 1.6.0 to ????
+       * 
+       *   there seem to be something strange about 
+       *   the XRecordCreateContext call
+       *   which causes the XRecordEnableContextAsync 
+       *   to fail ... ugly fix, but it works
+       *
+       */
+      fprintf(stderr, "Workaround: Creating context on data display instead of control \n");
+      fprintf(stderr, "            You can ignore this message\n");
+      context_display = xd->data ;
+    }
+  else
+    {
+      context_display = xd->control ;
+    }
+
+
+
+  xnee_verbose((xd, " --- xnee_record_async() enable context\n"));
+  ret = XRecordEnableContextAsync(context_display, 
+			     xd->record_setup->rContext, 
+			     xd->rec_callback, 
+			     (XPointer) (xd) /* closure passed to Dispatch */);
+
+
+  if (ret==0)
   {
      return (XNEE_RECORD_FAILURE);
   }
+  
 
 /*   XNEE_RETURN_IF_ERR(ret); */
 
-  
+
   for (;;) 
     {
+      xnee_verbose((xd, " --- xnee_record_async() loop\n"));
+
+
       /* Interrupt variable set? */
       if (xnee_is_interrupt_action(xd))
 	{
@@ -1053,7 +1174,6 @@ xnee_record_async(xnee_data *xd)
 	  break;
 	}
 
-
       /* has the user pressed a modifier+key */
       if (xnee_check_key (xd)==XNEE_GRAB_DATA)
 	{
@@ -1068,7 +1188,7 @@ xnee_record_async(xnee_data *xd)
 	  else if (ret == XNEE_GRAB_RESUME)
 	    {
 	      xnee_verbose  ((xd," starting async loop since RESUME \n"));
-	      ret = XRecordEnableContextAsync(xd->data, 
+	      ret = XRecordEnableContextAsync(xd->control, 
                                               xd->record_setup->rContext, 
                                               xd->rec_callback, 
                                               (XPointer) (xd) );
@@ -1080,11 +1200,12 @@ xnee_record_async(xnee_data *xd)
 	    }
 	}
 
+
       /* handle data in the RECORD buffer */
       ret = xnee_process_replies(xd);
       XNEE_RETURN_IF_ERR(ret);
 
-      if ( xnee_more_to_record(xd) == 0 ) 
+       if ( xnee_more_to_record(xd) == 0 ) 
 	{
 	  xnee_verbose  ((xd," closing down while loop in async loop\n"));
 	  ret=XNEE_OK;
