@@ -605,16 +605,6 @@ static xnee_option_t xnee_options_impl[] =
       XNEE_OPTION_VISIBLE
     },
 
-    {
-      XNEE_KEEP_AUTOREPEAT,
-      "keep-autorepeat",
-      "ka",
-      NULL,
-      "Keep autorepeat during record/replay",
-      XNEE_GENERAL_OPTION,
-      XNEE_OPTION_VISIBLE
-    },
-
     {
       XNEE_RETYPE_PRESS_DELAY,
       "retype-press-delay",
@@ -918,7 +908,9 @@ xnee_key2string(xnee_data      *xd,
   id = xnee_key2id(xd, options, key);
 
   xnee_verbose((xd, "Looking for %d ===> %d  '%s'\n",
-	  key, id, options[id].option));
+		key, 
+		id, 
+		EMPTY_IF_NULL( options[id].option)));
 
   if ( id == XNEE_OPTION_NOT_FOUND )
     {
@@ -1115,7 +1107,9 @@ xnee_parse_option_impl(xnee_data *xd, char **opt_and_args, int *args_used, int s
   key = xnee_options[entry].key;
 
   xnee_verbose((xd, "Found xns entry for '%s' '%s' at position: %d\n", 
-		opt_and_args[0],opt_and_args[1], entry));
+		EMPTY_IF_NULL(opt_and_args[0]),
+		EMPTY_IF_NULL(opt_and_args[1]), 
+		entry));
 
   xnee_verbose((xd, "\tlong option: '%u'\n", 
 		(unsigned int) xnee_options[entry].option));
