@@ -121,6 +121,13 @@ xnee_close_down(xnee_data* xd)
 
   xnee_verbose((xd, "xnee_close_down : ungrab -----> \n"));
   ret = xnee_ungrab_keys (xd);
+
+  if ( xd->grab_keys!= NULL) 
+  {
+      xnee_free(xd->grab_keys);
+  }
+  xd->grab_keys = NULL ;
+
   xnee_verbose((xd, "xnee_close_down : ungrab <---- %d\n", ret));
   xnee_verbose((xd, "xnee_close_down : xnee_reset_autorepeat ----> \n"));
   ret = xnee_reset_autorepeat (xd_global);
@@ -155,6 +162,8 @@ xnee_close_down(xnee_data* xd)
     {      
       XNEE_DEBUG ( (stderr ," --> xnee_close_down() at 0.2 \n"  ));
       xnee_verbose((xd, "%s\n", xd->display));
+      xnee_free(xd->display);
+      xd->display=NULL;
     }
 
   if ( xd->control!=NULL)  
@@ -221,6 +230,36 @@ xnee_close_down(xnee_data* xd)
 }
 
 
+int 
+xnee_set_x_server_version(xnee_data *xd)
+{
+  static int vendrel ;
+  static Display *dpy;
+  int ret_val = 0;
+
+  if (xd==NULL)
+    {
+      return XNEE_MEMORY_FAULT;
+    }
+
+  dpy = XOpenDisplay(xd->display);
+  if (dpy==NULL)
+    {
+      return XNEE_NOT_OPEN_DISPLAY;
+    }
+
+
+
+  xd->x_vendor_name = ServerVendor (dpy);
+  vendrel = VendorRelease(dpy);
+    
+  xd->x_version_major     =  vendrel / 10000000 ;
+  xd->x_version_minor     = (vendrel / 100000) % 100 ;
+  xd->x_version_minor_sub = (vendrel / 1000) % 100 ;
+
+  return XNEE_OK;
+}
+
 
 /**************************************************************
  *                                                            *
@@ -356,6 +395,8 @@ xnee_init(xnee_data* xd)
   ret = xnee_resolution_init (xd);
   XNEE_RETURN_IF_ERR(ret);
 
+  xnee_set_x_server_version(xd);
+
   /* Set the signal handler the libxnee's built in */ 
   (void) signal (SIGINT, signal_handler);
   
@@ -683,13 +724,21 @@ xnee_more_to_record(xnee_data *xd)
   int dats;
   int tims;
 
+  /*
+  fprintf (stderr, "  [ %d(%d) %d(%d) %d ] \n", 
+	   xnee_get_events_left(xd),
+	   xd->xnee_info.events_recorded,
+	   xnee_get_data_left(xd),
+	   xd->xnee_info.data_recorded,
+	   xnee_get_time_left(xd));
+  */
+
   evs = xnee_get_events_left(xd);
   if (evs<0) evs = 1;
 
   dats = xnee_get_data_left(xd);
   if (dats<0) dats = 1;
 
-
   xnee_record_update_time_left(xd);
   tims = xnee_get_time_left(xd);
   if (tims<0) tims = 1;
